@@ -1,20 +1,23 @@
 import sys
-def fix_sqlalchemy_StatementError_str():
+
+def fix_sqlalchemy_StatementError_str() -> None:
     # see https://github.com/sqlalchemy/sqlalchemy/issues/5632
-    import sqlalchemy # type: ignore
-    v = sqlalchemy.__version__
+    import sqlalchemy
+    v = sqlalchemy.__version__  # type: ignore[attr-defined]
     if v != '1.3.19':
         # sigh... will still affect smaller versions.. but patching code to remove import dynamically would be far too mad
         return
 
-    from sqlalchemy.util import compat # type: ignore
-    from sqlalchemy.exc import StatementError as SE # type: ignore
+    from sqlalchemy.util import compat
+    from sqlalchemy.exc import StatementError as SE
 
     def _sql_message(self, as_unicode):
         details = [self._message(as_unicode=as_unicode)]
         if self.statement:
-            if not as_unicode and not compat.py3k:
-                stmt_detail = "[SQL: %s]" % compat.safe_bytestring(
+            # pylint: disable=no-member
+            if not as_unicode and not compat.py3k:  # type: ignore[attr-defined]
+                # pylint: disable=no-member
+                stmt_detail = "[SQL: %s]" % compat.safe_bytestring(  # type: ignore[attr-defined]
                     self.statement
                 )
             else:
@@ -27,9 +30,9 @@ def fix_sqlalchemy_StatementError_str():
                     )
                 else:
                     # NOTE: this will still cause issues
-                    from sqlalchemy.sql import util # type: ignore
+                    from sqlalchemy.sql import util
 
-                    params_repr = util._repr_params(
+                    params_repr = util._repr_params(  # type: ignore[attr-defined]
                         self.params, 10, ismulti=self.ismulti
                     )
                     details.append("[parameters: %r]" % params_repr)
@@ -38,4 +41,4 @@ def fix_sqlalchemy_StatementError_str():
             details.append(code_str)
         return "\n".join(["(%s)" % det for det in self.detail] + details)
 
-    SE._sql_message = _sql_message
+    SE._sql_message = _sql_message  # type: ignore[attr-defined,assignment]
